@@ -20,11 +20,12 @@
  */
 
 #include "servercore.h"
-#include "serverconnection.h"
-#include "dbus/dbusserveradaptor.h"
-#include "modelpool.h"
-
 #include "soprano-server-config.h"
+#include "serverconnection.h"
+#ifdef BUILD_DBUS_SUPPORT
+#include "dbus/dbusserveradaptor.h"
+#endif
+#include "modelpool.h"
 
 #include "backend.h"
 #include "storagemodel.h"
@@ -52,9 +53,12 @@ class Soprano::Server::ServerCore::Private
 {
 public:
     Private()
-        : dbusAdaptor( 0 ),
-          tcpServer( 0 ),
-          socketServer( 0 )
+        :
+#ifdef BUILD_DBUS_SUPPORT
+        dbusAdaptor( 0 ),
+#endif
+        tcpServer( 0 ),
+        socketServer( 0 )
     {}
 
     const Backend* backend;
@@ -63,7 +67,9 @@ public:
     QHash<QString, Model*> models;
     QList<ServerConnection*> connections;
 
+#ifdef BUILD_DBUS_SUPPORT
     DBusServerAdaptor* dbusAdaptor;
+#endif
 
     QTcpServer* tcpServer;
     QLocalServer* socketServer;
@@ -73,12 +79,9 @@ public:
 
     BackendSettings createBackendSettings( const QString& name ) {
         BackendSettings newSettings = settings;
-        for ( BackendSettings::iterator it = newSettings.begin();
-              it != newSettings.end(); ++it ) {
-            BackendSetting& setting = *it;
-            if ( setting.option() == BackendOptionStorageDir ) {
-                setting.setValue( setting.value().toString() + '/' + name );
-            }
+        if ( isOptionInSettings( newSettings, BackendOptionStorageDir ) ) {
+            BackendSetting& setting = settingInSettings( newSettings, BackendOptionStorageDir );
+            setting.setValue( setting.value().toString() + '/' + name );
         }
         return newSettings;
     }
@@ -151,16 +154,11 @@ QList<Soprano::BackendSetting> Soprano::Server::ServerCore::backendSettings() co
 
 Soprano::Model* Soprano::Server::ServerCore::model( const QString& name )
 {
-    QHash<QString, Model*>::const_iterator it = d->models.find( name );
+    QHash<QString, Model*>::const_iterator it = d->models.constFind( name );
     if ( it == d->models.constEnd() ) {
-        QList<BackendSetting> settings = d->createBackendSettings( name );
-        for ( QList<BackendSetting>::iterator it = settings.begin();
-              it != settings.end(); ++it ) {
-            BackendSetting& setting = *it;
-            if ( setting.option() == BackendOptionStorageDir ) {
-                QDir().mkpath( setting.value().toString() );
-                break;
-            }
+        BackendSettings settings = d->createBackendSettings( name );
+        if ( isOptionInSettings( settings, BackendOptionStorageDir ) ) {
+            QDir().mkpath( valueInSettings( settings, BackendOptionStorageDir ).toString() );
         }
 
         Model* model = createModel( settings );
@@ -185,7 +183,7 @@ void Soprano::Server::ServerCore::removeModel( const QString& name )
     clearError();
 
     QHash<QString, Model*>::iterator it = d->models.find( name );
-    if ( it == d->models.constEnd() ) {
+    if ( it == d->models.end() ) {
         setError( QString( "Could not find model with name %1" ).arg( name ) );
     }
     else {
@@ -194,17 +192,12 @@ void Soprano::Server::ServerCore::removeModel( const QString& name )
         // delete the model, removing any cached data
         delete model;
 
-        // remove the data on disk
-        backend()->deleteModelData( d->createBackendSettings( name ) );
+        if ( isOptionInSettings( d->settings, BackendOptionStorageDir ) ) {
+            // remove the data on disk
+            backend()->deleteModelData( d->createBackendSettings( name ) );
 
-        // remove the dir which should now be empty
-        for ( QList<BackendSetting>::iterator it = d->settings.begin();
-              it != d->settings.end(); ++it ) {
-            BackendSetting& setting = *it;
-            if ( setting.option() == BackendOptionStorageDir ) {
-                QDir( setting.value().toString() ).rmdir( name );
-                break;
-            }
+            // remove the dir which should now be empty
+            QDir( valueInSettings( d->settings, BackendOptionStorageDir ).toString() ).rmdir( name );
         }
     }
 }
@@ -268,6 +261,7 @@ bool Soprano::Server::ServerCore::start( const QString& name )
 
 void Soprano::Server::ServerCore::registerAsDBusObject( const QString& objectPath )
 {
+#ifdef BUILD_DBUS_SUPPORT
     if ( !d->dbusAdaptor ) {
         QString path( objectPath );
         if ( path.isEmpty() ) {
@@ -277,6 +271,9 @@ void Soprano::Server::ServerCore::registerAsDBusObject( const QString& objectPat
         d->dbusAdaptor = new Soprano::Server::DBusServerAdaptor( this, path );
         QDBusConnection::sessionBus().registerObject( path, this );
     }
+#else
+    qFatal("Soprano has been built without D-Bus support!" );
+#endif
 }
 
 
