@@ -1,7 +1,7 @@
 /*
  * This file is part of Soprano Project.
  *
- * Copyright (C) 2007-2008 Sebastian Trueg <trueg@kde.org>
+ * Copyright (C) 2007-2009 Sebastian Trueg <trueg@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,114 +24,49 @@
 #include "backend.h"
 #include "parser.h"
 #include "serializer.h"
-#include "query/queryparser.h"
-#include "query/queryserializer.h"
 #include "soprano-config.h"
 #include "sopranopluginfile.h"
 #include "version.h"
+#include "sopranodirs.h"
 
 #include <QtCore/QHash>
 #include <QtCore/QDir>
 #include <QtCore/QDebug>
 #include <QtCore/QMutex>
-#include <QtCore/QLibrary>
-
-#include <stdlib.h>
-
-#if defined _WIN32 || defined _WIN64
-#define PATH_SEPARATOR ';'
-#else
-#define PATH_SEPARATOR ':'
-#endif
+#include <QtCore/QCoreApplication>
 
 
 namespace {
-    inline QStringList makeLibName( const QString &libname )
-    {
-        int pos = libname.lastIndexOf('/');
-        if (pos < 0) {
-            pos = 0;
-        }
-
-        QStringList libNames;
-        if (libname.indexOf('.', pos) < 0) {
-            const char* const extList[] = { ".so", ".dylib", ".bundle", ".sl", ".dll" };
-            for (uint i = 0; i < sizeof(extList) / sizeof(*extList); ++i) {
-                if (QLibrary::isLibrary(libname + extList[i])) {
-                    libNames.append(libname + extList[i]);
-                }
-            }
-        }
-        else {
-            libNames.append(libname);
-        }
-        return libNames;
-    }
-
-    QStringList envDirList( const char* var )
-    {
-        QStringList dirs;
-        QByteArray varData = qgetenv( var );
-        if ( !varData.isEmpty() ) {
-            QStringList d = QString::fromLocal8Bit( varData ).split( PATH_SEPARATOR );
-            Q_FOREACH( const QString &dir, d ) {
-                dirs += QDir::fromNativeSeparators( dir );
-            }
-        }
-        return dirs;
-    }
-
-    QStringList dataDirs()
-    {
-        QStringList paths;
-
-        paths += QLatin1String(SOPRANO_PREFIX"/share");
-        paths += envDirList( "SOPRANO_DIRS" );
-        paths += envDirList( "XDG_DATA_DIRS" );
-
-        return paths;
-    }
-
-    /**
-     * lib dirs without lib suffix or trailing slash.
-     */
-    QStringList libDirs()
-    {
-        QStringList paths;
-        paths << QLatin1String( SOPRANO_PREFIX"/lib" );
-        paths << QLatin1String( "/usr/lib" );
-        paths << QLatin1String( "/usr/local/lib" );
+    QString findPluginLib( const Soprano::SopranoPluginFile& file ) {
+        QStringList fileSearchPaths;
+        // the folder the plugin file is in
+        fileSearchPaths << file.fileName().section( '/', 0, -2, QString::SectionIncludeTrailingSep );
 #ifndef Q_OS_WIN
-        paths += envDirList( "LD_LIBRARY_PATH" );
+        // the lib folder in the same prefix
+        fileSearchPaths << file.fileName().section( "/", 0, -5, QString::SectionIncludeTrailingSep ) + QLatin1String( "lib" );
 #endif
-        return paths;
+        return Soprano::findLibraryPath( file.library(), fileSearchPaths, QStringList() << QLatin1String( "soprano" ) );
     }
 
-    QString findPluginLib( const Soprano::SopranoPluginFile& file )
-    {
-        QStringList libs = makeLibName( file.library() );
-        Q_FOREACH( const QString &lib, libs ) {
-            if ( lib.startsWith( '/' ) ) {
-                return lib;
-            }
-            else {
-                QStringList dirs( libDirs() );
-                dirs.prepend( file.fileName().section( "/", 0, -5, QString::SectionIncludeTrailingSep ) + "lib" );
-
-                foreach( QString dir, dirs ) {
-                    if ( QFile::exists( dir + SOPRANO_LIB_SUFFIX"/soprano/" + lib ) ) {
-                        return dir + SOPRANO_LIB_SUFFIX"/soprano/" + lib;
-                    }
-                    else if ( QFile::exists( dir + "/soprano/" + lib ) ) {
-                        return dir + "/soprano/" + lib;
-                    }
-                    else if ( QFile::exists( dir + "64/soprano/" + lib ) ) {
-                        return dir + "64/soprano/" + lib;
-                    }
-                }
-            }
+    QStringList pluginFileSearchPaths() {
+        QStringList searchPaths;
+        foreach( const QString& p, Soprano::dataDirs() ) {
+            searchPaths << ( p + QLatin1String( "/soprano/plugins" ) );
         }
-        return QString();
+#ifdef Q_OS_WIN
+        // when used as a standalone lib on windows,
+        // Soprano is not installed in a default location as on linux/unix systems.
+        // Thus, we search in some well known folders for plugins
+        QStringList winSearchPaths;
+        winSearchPaths << QCoreApplication::applicationDirPath()
+                       << QCoreApplication::libraryPaths();
+        foreach( const QString& p, winSearchPaths ) {
+            searchPaths << p + QLatin1String( "/soprano" );
+            searchPaths << p + QLatin1String( "/plugins" );
+        }
+#endif
+
+        return searchPaths;
     }
 }
 
@@ -143,6 +78,8 @@ public:
         : pluginsLoaded(false) {
     }
 
+    bool addPlugin( PluginStub& stub );
+
     bool pluginsLoaded;
 
     QHash<QString, PluginStub> backends;
@@ -150,13 +87,42 @@ public:
     QHash<QString, PluginStub> serializers;
     QHash<QString, PluginStub> queryParsers;
     QHash<QString, PluginStub> querySerializers;
+
+    QStringList searchPaths;
+    bool useDefaultSearchPaths;
 };
 
 
+bool Soprano::PluginManager::Private::addPlugin( PluginStub& stub )
+{
+    if ( QObject* p = stub.plugin() ) {
+        if ( qobject_cast<Soprano::Backend*>( p ) ) {
+            backends.insert( stub.name(), stub );
+        }
+        else if ( qobject_cast<Soprano::Parser*>( p ) ) {
+            parsers.insert( stub.name(), stub );
+        }
+        else if ( qobject_cast<Soprano::Serializer*>( p ) ) {
+            serializers.insert( stub.name(), stub );
+        }
+        else {
+            qDebug() << "Plugin" << stub.name() << "has unknown plugin type";
+            return false;
+        }
+
+        return true;
+    }
+    else {
+        return false;
+    }
+}
+
+
 Soprano::PluginManager::PluginManager( QObject* parent )
     : QObject( parent ),
       d( new Private )
 {
+    setPluginSearchPath( QStringList(), true );
 }
 
 
@@ -171,7 +137,11 @@ const Soprano::Backend* Soprano::PluginManager::discoverBackendByName( const QSt
     loadAllPlugins();
     QHash<QString, PluginStub>::iterator it = d->backends.find( name );
     if( it != d->backends.end() )
-        return dynamic_cast<Backend*>( it->plugin() );
+        return qobject_cast<Backend*>( it->plugin() );
+    else if ( !name.endsWith( QLatin1String( "backend" ) ) )
+        return discoverBackendByName( name + QLatin1String( "backend" ) );
+    else if ( !name.endsWith( QLatin1String( "backend" ) ) )
+        return discoverBackendByName( name + QLatin1String( "backend" ) );
     else
         return 0;
 }
@@ -181,7 +151,7 @@ const Soprano::Backend* Soprano::PluginManager::discoverBackendByFeatures( Backe
 {
     loadAllPlugins();
     for( QHash<QString, PluginStub>::iterator it = d->backends.begin(); it != d->backends.end(); ++it ) {
-        if( const Backend* b = dynamic_cast<Backend*>( it->plugin() ) ) {
+        if( const Backend* b = qobject_cast<Backend*>( it->plugin() ) ) {
             if( b->supportsFeatures( features, userFeatures ) ) {
                 return b;
             }
@@ -196,7 +166,11 @@ const Soprano::Parser* Soprano::PluginManager::discoverParserByName( const QStri
     loadAllPlugins();
     QHash<QString, PluginStub>::iterator it = d->parsers.find( name );
     if( it != d->parsers.end() )
-        return dynamic_cast<Parser*>( it->plugin() );
+        return qobject_cast<Parser*>( it->plugin() );
+    else if ( !name.endsWith( QLatin1String( "parser" ) ) )
+        return discoverParserByName( name + QLatin1String( "parser" ) );
+    else if ( !name.endsWith( QLatin1String( "parser" ) ) )
+        return discoverParserByName( name + QLatin1String( "parser" ) );
     else
         return 0;
 }
@@ -206,7 +180,7 @@ const Soprano::Parser* Soprano::PluginManager::discoverParserForSerialization( R
 {
     loadAllPlugins();
     for( QHash<QString, PluginStub>::iterator it = d->parsers.begin(); it != d->parsers.end(); ++it ) {
-        if( const Parser* p = dynamic_cast<Parser*>( it->plugin() ) ) {
+        if( const Parser* p = qobject_cast<Parser*>( it->plugin() ) ) {
             if( p->supportsSerialization( serialization, userSerialization ) ) {
                 return p;
             }
@@ -221,7 +195,11 @@ const Soprano::Serializer* Soprano::PluginManager::discoverSerializerByName( con
     loadAllPlugins();
     QHash<QString, PluginStub>::iterator it = d->serializers.find( name );
     if( it != d->serializers.end() )
-        return dynamic_cast<Serializer*>( it->plugin() );
+        return qobject_cast<Serializer*>( it->plugin() );
+    else if ( !name.endsWith( QLatin1String( "serializer" ) ) )
+        return discoverSerializerByName( name + QLatin1String( "serializer" ) );
+    else if ( !name.endsWith( QLatin1String( "serializer" ) ) )
+        return discoverSerializerByName( name + QLatin1String( "serializer" ) );
     else
         return 0;
 }
@@ -231,7 +209,7 @@ const Soprano::Serializer* Soprano::PluginManager::discoverSerializerForSerializ
 {
     loadAllPlugins();
     for( QHash<QString, PluginStub>::iterator it = d->serializers.begin(); it != d->serializers.end(); ++it ) {
-        if( const Serializer* p = dynamic_cast<Serializer*>( it->plugin() ) ) {
+        if( const Serializer* p = qobject_cast<Serializer*>( it->plugin() ) ) {
             if( p->supportsSerialization( serialization, userSerialization ) ) {
                 return p;
             }
@@ -247,7 +225,7 @@ QList<const Soprano::Backend*> Soprano::PluginManager::allBackends()
     QList<const Backend*> bl;
     for ( QHash<QString, PluginStub>::iterator it = d->backends.begin();
           it != d->backends.end(); ++it ) {
-        if( const Backend* p = dynamic_cast<Backend*>( it->plugin() ) ) {
+        if( const Backend* p = qobject_cast<Backend*>( it->plugin() ) ) {
             bl.append( p );
         }
     }
@@ -261,7 +239,7 @@ QList<const Soprano::Parser*> Soprano::PluginManager::allParsers()
     QList<const Parser*> pl;
     for ( QHash<QString, PluginStub>::iterator it = d->parsers.begin();
           it != d->parsers.end(); ++it ) {
-        if( const Parser* p = dynamic_cast<Parser*>( it->plugin() ) ) {
+        if( const Parser* p = qobject_cast<Parser*>( it->plugin() ) ) {
             pl.append( p );
         }
     }
@@ -275,7 +253,7 @@ QList<const Soprano::Serializer*> Soprano::PluginManager::allSerializers()
     QList<const Serializer*> pl;
     for ( QHash<QString, PluginStub>::iterator it = d->serializers.begin();
           it != d->serializers.end(); ++it ) {
-        if( const Serializer* p = dynamic_cast<Serializer*>( it->plugin() ) ) {
+        if( const Serializer* p = qobject_cast<Serializer*>( it->plugin() ) ) {
             pl.append( p );
         }
     }
@@ -286,14 +264,18 @@ QList<const Soprano::Serializer*> Soprano::PluginManager::allSerializers()
 void Soprano::PluginManager::loadAllPlugins()
 {
     if( !d->pluginsLoaded ) {
-        qDebug() << "(Soprano::PluginManager) loading all plugins";
+//        qDebug() << "(Soprano::PluginManager) loading all plugins";
 
-        QStringList dirs = dataDirs();
-        Q_FOREACH( const QString &dir, dirs ) {
-            QDir pluginDir( dir + "/soprano/plugins" );
-            qDebug() << "(Soprano::PluginManager) searching plugin file from " << pluginDir.absolutePath();
+        QStringList searchPaths = d->searchPaths;
+        if ( d->useDefaultSearchPaths ) {
+            searchPaths << pluginFileSearchPaths();
+        }
+
+        Q_FOREACH( const QString& dir, searchPaths ) {
+            QDir pluginDir( dir );
+//            qDebug() << "(Soprano::PluginManager) searching plugin file from " << pluginDir.absolutePath();
             QStringList pluginFiles = pluginDir.entryList( QStringList( QLatin1String( "*.desktop" ) ) );
-            Q_FOREACH( QString plugin, pluginFiles ) {
+            Q_FOREACH( const QString& plugin, pluginFiles ) {
                 loadPlugin( pluginDir.absoluteFilePath( plugin ) );
             }
         }
@@ -306,10 +288,15 @@ void Soprano::PluginManager::loadPlugin( const QString& path )
 {
     SopranoPluginFile f;
     if ( f.open( path ) ) {
-        qDebug() << "(Soprano::PluginManager) found plugin file" << path;
+//        qDebug() << "(Soprano::PluginManager) found plugin file" << path;
         if ( f.sopranoVersion().left( f.sopranoVersion().indexOf( '.' ) ).toUInt() == Soprano::versionMajor() ) {
-            qDebug() << "(Soprano::PluginManager) plugin has proper version.";
+//            qDebug() << "(Soprano::PluginManager) plugin has proper version.";
+
             QString libPath = findPluginLib( f );
+            if ( libPath.isEmpty() ) {
+                qDebug() << "Failed to find the library for plugin" << f.pluginName() << "(" << f.library() << ")";
+                return;
+            }
 
             QString type = f.serviceTypes();
             QString name = f.pluginName();
@@ -318,44 +305,37 @@ void Soprano::PluginManager::loadPlugin( const QString& path )
                 if( name.endsWith( ".desktop" ) ) {
                     name.truncate( name.length() - 8 );
                 }
-
-                // try some backwards compatibility
-                // as backends redland and sesame2 were always loaded without
-                // the "backend" suffix
-                if( name.endsWith( "backend" ) ) {
-                    name.truncate( name.length() - 7 );
-                }
             }
 
             PluginStub plugin( name, libPath );
 
             if( type.contains( "Soprano/Backend" ) ) {
                 if ( !d->backends.contains( name ) ) {
-                    qDebug() << "(Soprano::PluginManager) found backend plugin " << name;
+//                    qDebug() << "(Soprano::PluginManager) found backend plugin " << name;
                     d->backends.insert( name, plugin );
                 }
             }
             else if( type.contains( "Soprano/Parser" ) ) {
                 if ( !d->parsers.contains( name ) ) {
-                    qDebug() << "(Soprano::PluginManager) found parser plugin " << name;
+//                    qDebug() << "(Soprano::PluginManager) found parser plugin " << name;
                     d->parsers.insert( name, plugin );
                 }
             }
             else if( type.contains( "Soprano/Serializer" ) ) {
                 if ( !d->serializers.contains( name ) ) {
-                    qDebug() << "(Soprano::PluginManager) found serializer plugin " << name;
+//                    qDebug() << "(Soprano::PluginManager) found serializer plugin " << name;
                     d->serializers.insert( name, plugin );
                 }
             }
             else if( type.contains( "Soprano/QueryParser" ) ) {
                 if ( !d->queryParsers.contains( name ) ) {
-                    qDebug() << "(Soprano::PluginManager) found query parser plugin " << name;
+//                    qDebug() << "(Soprano::PluginManager) found query parser plugin " << name;
                     d->queryParsers.insert( name, plugin );
                 }
             }
             else if( type.contains( "Soprano/QuerySerializer" ) ) {
                 if ( !d->querySerializers.contains( name ) ) {
-                    qDebug() << "(Soprano::PluginManager) found query serializer plugin " << name;
+//                    qDebug() << "(Soprano::PluginManager) found query serializer plugin " << name;
                     d->querySerializers.insert( name, plugin );
                 }
             }
@@ -366,22 +346,36 @@ void Soprano::PluginManager::loadPlugin( const QString& path )
         else {
         	   qDebug() << "(Soprano::PluginManager) plugin has major version:"
         	            << f.sopranoVersion().left( f.sopranoVersion().indexOf( '.' ) ).toUInt()
-                     << "required major version:" << Soprano::versionMajor();
+                        << "required major version:" << Soprano::versionMajor();
         }
     }
 }
 
 
+void Soprano::PluginManager::setPluginSearchPath( const QStringList& paths, bool useDefaults )
+{
+    d->searchPaths = paths;
+    d->useDefaultSearchPaths = useDefaults;
+}
+
+
+namespace Soprano {
+    /**
+     * Little hack class to ensure BC (cannot make PluginManager constructor public)
+     * and at the same time use Q_GLOBAL_STATIC
+     */
+    class PluginManagerFactory
+    {
+    public:
+        Soprano::PluginManager manager;
+    };
+}
+
+Q_GLOBAL_STATIC( Soprano::PluginManagerFactory, s_pluginManagerFactory )
+
 Soprano::PluginManager* Soprano::PluginManager::instance()
 {
-    static PluginManager* s_instance = 0;
-    static QMutex s_mutex;
-    s_mutex.lock();
-    if( !s_instance ) {
-        s_instance = new PluginManager();
-    }
-    s_mutex.unlock();
-    return s_instance;
+    return &s_pluginManagerFactory()->manager;
 }
 
 #include "pluginmanager.moc"
