@@ -27,8 +27,6 @@
 #include <QtTest/QTest>
 #include <QtCore/QList>
 #include <QtCore/QDebug>
-#include <QtCore/QFile>
-#include <QtCore/QTextStream>
 #include <QtCore/QTime>
 #include <QtCore/QUrl>
 #include <QtCore/QUuid>
@@ -64,7 +62,9 @@ static QList<Statement> createTestData( const Statement& s, int num )
 
 
 SopranoModelTest::SopranoModelTest()
-    : m_model( 0 )
+    : m_model( 0 ),
+      m_features( BackendFeatureAll ),
+      m_testSignals( true )
 {
 }
 
@@ -129,9 +129,9 @@ void SopranoModelTest::testAddListOfStatement()
 
     m_model->addStatements( statements );
 
-    QVERIFY( m_model->containsAnyStatement( st1 ) );
-    QVERIFY( m_model->containsAnyStatement( st2 ) );
-    QVERIFY( m_model->containsAnyStatement( st3 ) );
+    QVERIFY( m_model->containsStatement( st1 ) );
+    QVERIFY( m_model->containsStatement( st2 ) );
+    QVERIFY( m_model->containsStatement( st3 ) );
 }
 
 
@@ -212,10 +212,6 @@ void SopranoModelTest::testListStatements()
 
     StatementIterator it1 = m_model->listStatements( Statement( resource_1, Node(), Node() ) );
 
-    QFile f( "/tmp/testdump" );
-    f.open( QIODevice::WriteOnly );
-    QTextStream fs( &f );
-
     int cnt = 0;
     while( it1.next() ) {
         ++cnt;
@@ -225,8 +221,6 @@ void SopranoModelTest::testListStatements()
             qDebug() << st.subject() << "vs." << resource_1;
         }
         QCOMPARE( st.subject(), resource_1 );
-
-        fs << st << endl;
     }
 
     QCOMPARE( cnt, 50 );
@@ -439,6 +433,13 @@ void SopranoModelTest::testContainsStatement()
     QVERIFY( m_model->containsStatement( Statement( res1, pre, res4, context2 ) ) );
     QVERIFY( !m_model->lastError() );
 
+    QVERIFY( !m_model->containsStatement( Statement( context2, res1, pre, res3 ) ) );
+    QVERIFY( !m_model->lastError() );
+    QVERIFY( !m_model->containsStatement( Statement( res1, pre, res4, context1 ) ) );
+    QVERIFY( !m_model->lastError() );
+    QVERIFY( !m_model->containsStatement( Statement( res1, pre, res3, context2 ) ) );
+    QVERIFY( !m_model->lastError() );
+
     // make sure the context is not used as wildcard
     QVERIFY( !m_model->containsStatement( Statement( res1, pre, res3 ) ) );
     QVERIFY( !m_model->lastError() );
@@ -480,6 +481,9 @@ void SopranoModelTest::testContainsStatements()
 
 void SopranoModelTest::testGraphQuery()
 {
+    if ( !( m_features&BackendFeatureQuery ) )
+        return;
+
     QVERIFY( m_model != 0 );
 
     QString query( "CONSTRUCT { ?s ?p ?o } WHERE { ?s ?p ?o . }" );
@@ -499,6 +503,9 @@ void SopranoModelTest::testGraphQuery()
 
 void SopranoModelTest::testBooleanQuery()
 {
+    if ( !( m_features&BackendFeatureQuery ) )
+        return;
+
     QVERIFY( m_model != 0 );
 
     QString query( "ASK where {?a ?b ?c}" );
@@ -523,6 +530,9 @@ void SopranoModelTest::testBooleanQuery()
 
 void SopranoModelTest::testInvalidQuery()
 {
+    if ( !( m_features&BackendFeatureQuery ) )
+        return;
+
     QVERIFY( m_model != 0 );
 
     QueryResultIterator res = m_model->executeQuery( "INVALID query", Query::QueryLanguageSparql );
@@ -531,6 +541,9 @@ void SopranoModelTest::testInvalidQuery()
 
 void SopranoModelTest::testQuery()
 {
+    if ( !( m_features&BackendFeatureQuery ) )
+        return;
+
     QVERIFY( m_model != 0 );
 
     /* SPARQL */
@@ -832,6 +845,10 @@ void SopranoModelTest::testLiteralTypes_data()
 
     QString ns = "http://soprano.org/literalTest#";
 
+    // plain literals
+    QTest::newRow("plain") << LiteralValue::createPlainLiteral( "Hello World" ) << QUrl(ns + "plain");
+    QTest::newRow("plainDe") << LiteralValue::createPlainLiteral( "Hallo Welt", "de" ) << QUrl(ns + "plainDe");
+
     // all the standard literal types
     QTest::newRow("intValue") << LiteralValue( 42 ) << QUrl(ns + "intValue");
     QTest::newRow("boolFalseValue") << LiteralValue( false ) << QUrl(ns + "boolFalseValue");
@@ -847,6 +864,16 @@ void SopranoModelTest::testLiteralTypes_data()
     QTest::newRow("stringValueWithGermanUmlauts") << LiteralValue( StringPool::germanUmlauts() ) << QUrl(ns + "stringValueWithGermanUmlauts");
     QTest::newRow("stringValueWithFrenchAccents") << LiteralValue( StringPool::frenchAccents() ) << QUrl(ns + "stringValueWithFrenchAccents");
     QTest::newRow("stringValueWithRussianChars") << LiteralValue( StringPool::russianChars() ) << QUrl(ns + "stringValueWithRussianChars");
+
+    // and some others
+    QTest::newRow("stringWithSingleQuote" ) << LiteralValue( QString( "c'est" ) ) << QUrl( ns + "stringWithSingleQuote" );
+    QTest::newRow("stringWith2SingleQuote" ) << LiteralValue( QString( "\'\'real" ) ) << QUrl( ns + "stringWith2SingleQuote" );
+    QTest::newRow("stringWith3SingleQuote" ) << LiteralValue( QString( "\'\'\'real" ) ) << QUrl( ns + "stringWith3SingleQuote" );
+    QTest::newRow("stringWithDoubleQuote" ) << LiteralValue( QString( "\"real" ) ) << QUrl( ns + "stringWithDoubleQuote" );
+    QTest::newRow("stringWith2DoubleQuote" ) << LiteralValue( QString( "\"\"real" ) ) << QUrl( ns + "stringWith2DoubleQuote" );
+    QTest::newRow("stringWith3DoubleQuote" ) << LiteralValue( QString( "\"\"\"real" ) ) << QUrl( ns + "stringWith3DoubleQuote" );
+    QTest::newRow("stringWithTab" ) << LiteralValue( QString( "foo\tbar" ) ) << QUrl( ns + "stringWithTab" );
+    QTest::newRow("stringWithNewLine" ) << LiteralValue( QString( "foo\nbar" ) ) << QUrl( ns + "stringWithNewLine" );
 }
 
 
@@ -867,6 +894,10 @@ void SopranoModelTest::testLiteralTypes()
     QVERIFY( it.next() );
     QCOMPARE( it.current().object().literal(), literal );
     it.close();
+
+    it = m_model->listStatements( Statement( Node(), Node(), literal ) );
+    QVERIFY( it.next() );
+    QCOMPARE( it.current().object().literal(), literal );
 }
 
 
@@ -887,6 +918,8 @@ void SopranoModelTest::testUriEncoding_data()
     QTest::newRow("germanUmlauts") << QUrl( ns + StringPool::germanUmlauts() ) << QUrl(ns + "germanUmlauts");
     QTest::newRow("frenchAccents") << QUrl( ns + StringPool::frenchAccents() ) << QUrl(ns + "frenchAccents");
     QTest::newRow("russianChars") << QUrl( ns + StringPool::russianChars() ) << QUrl(ns + "russianChars");
+    QTest::newRow("withSingleQuote" ) << QUrl( ns + "c'est" ) << QUrl( ns + "singleQuote" );
+    QTest::newRow("withDoubleQuote" ) << QUrl( ns + "\"real" ) << QUrl( ns + "doubleQuote" );
 }
 
 
@@ -919,6 +952,8 @@ void SopranoModelTest::testIteratorNesting()
 {
     QVERIFY( m_model );
 
+    m_model->removeAllStatements();
+
     Node c1 = createRandomUri();
     Node c2 = createRandomUri();
     Node c3 = createRandomUri();
@@ -972,6 +1007,26 @@ void SopranoModelTest::testIteratorNesting()
             QCOMPARE( cnt1, 5 );
         }
     }
+
+
+    if ( !( m_features&BackendFeatureQuery ) )
+        return;
+
+    // test query iterator nesting
+    cnt1 = 0;
+    QueryResultIterator it4 = m_model->executeQuery( "select * where { graph ?g { ?r ?p ?o . } . }",
+                                                     Query::QueryLanguageSparql );
+    while ( it4.next() ) {
+        ++cnt1;
+        QUrl g = it4["g"].uri();
+        QueryResultIterator it5 = m_model->executeQuery( QString( "select * where { graph %1 { ?r ?p ?o . } . }" ).arg( Node::resourceToN3( g ) ),
+                                                         Query::QueryLanguageSparql );
+        cnt2 = 0;
+        while ( it5.next() )
+            ++cnt2;
+        QCOMPARE( cnt2, 5 );
+    }
+    QCOMPARE( cnt1, 20 );
 }
 
 
@@ -990,6 +1045,9 @@ namespace {
 
 void SopranoModelTest::testStatementsAddedSignal()
 {
+    if ( !m_testSignals )
+        return;
+
     QVERIFY( m_model );
 
     m_model->removeAllStatements();
@@ -1010,6 +1068,9 @@ void SopranoModelTest::testStatementsAddedSignal()
 
 void SopranoModelTest::testStatementAddedSignal()
 {
+    if ( !m_testSignals )
+        return;
+
     QVERIFY( m_model );
 
     m_model->removeAllStatements();
@@ -1037,6 +1098,9 @@ void SopranoModelTest::testStatementAddedSignal()
 
 void SopranoModelTest::testStatementsRemovedSignal()
 {
+    if ( !m_testSignals )
+        return;
+
     QVERIFY( m_model );
 
     m_model->removeAllStatements();
@@ -1066,6 +1130,9 @@ void SopranoModelTest::testStatementsRemovedSignal()
 
 void SopranoModelTest::testStatementRemovedSignal()
 {
+    if ( !m_testSignals )
+        return;
+
     QVERIFY( m_model );
 
     m_model->removeAllStatements();
